import 'dart:io';

import 'package:contacts_service/contacts_service.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:share_apps/model/user_bean.dart';
import 'package:share_apps/screens/home_screen.dart';
import 'package:share_apps/utils/color_util.dart';
import 'package:share_apps/utils/common_utils.dart';
import 'package:share_apps/utils/string_utils.dart';
import 'package:share_apps/widgets/widgets_importer.dart';

import 'invite_contact_screen.dart';

class ContactSelection extends StatefulWidget {
  ContactSelection(this.bean);

  final UserBean bean;

  @override
  _ContactSelectionState createState() => new _ContactSelectionState();
}

class _ContactSelectionState extends State<ContactSelection> {
  List<Contact> _contacts = new List<Contact>();
  List<CustomContact> _uiCustomContacts = List<CustomContact>();
  List<CustomContact> _allContacts = List<CustomContact>();
  List<CustomContact> _filterdContacts = List<CustomContact>();
  bool _isLoading = false;

  @override
  void initState() {
    super.initState();
    refreshContacts();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        leading: IconButton(
            icon: Icon(
              Icons.arrow_back_ios,
              color: Colors.black,
            ),
            onPressed: () {
              Navigator.pushReplacement(
                  context,
                  MaterialPageRoute(
                      builder: (context) => HomeScreen(
                            widget.bean,
                          )));
            }),
        elevation: 0,
        backgroundColor: Colors.white,
        title: Row(
          children: [
            Expanded(
              child: Center(
                child: AddText(StringUtils.CONTACT_SELECTION_TITLE, Colors.black, 18,
                    FontWeight.normal, false),
              ),
            ),
            IconButton(
                icon: Icon(
                  Icons.arrow_back_ios,
                  color: Colors.transparent,
                ),
            )
          ],
        ),
      ),
      bottomNavigationBar: GestureDetector(
        onTap: () {
          proceed();
        },
        child: Container(
            color: Colors.green,
            child: Padding(
              padding: const EdgeInsets.all(18.0),
              child: AddText(
                  StringUtils.DONE, Colors.white, 16, FontWeight.bold, false),
            )),
      ),
      body: !_isLoading
          ? Column(
              children: <Widget>[
                Container(
                  color: Colors.grey,
                  height: 0.5,
                ),
                Center(
                  child: Padding(
                    padding: const EdgeInsets.all(7.0),
                    child: new TextField(
                      textAlign: TextAlign.left,
                      decoration: new InputDecoration(
                          prefixIcon: Icon(
                            Icons.search,
                            color: Colors.grey,
                          ),
                          hintText: StringUtils.SEARCH_NUMBER_HINT,
                          hintStyle: TextStyle(color: Colors.black),
                          border: InputBorder.none),
                      onChanged: onSearchTextChanged,
                    ),
                  ),
                ),
                Container(
                  color: Colors.grey,
                  height: 0.5,
                ),
                Row(
                  children: <Widget>[
                    AddTopButton(false),
                    Container(
                      color: Colors.grey,
                      width: 0.5,
                      height: 50,
                    ),
                    AddTopButton(true),
                  ],
                ),
                Container(
                  color: Colors.grey,
                  height: 0.5,
                ),
                Expanded(
                  child: ListView.builder(
                    itemCount: _filterdContacts?.length,
                    itemBuilder: (BuildContext context, int index) {
                      CustomContact _contact = _filterdContacts[index];
                      var _phonesList = _contact.contact.phones.toList();
                      return Column(
                        children: [
                          _buildListTile(_contact, _phonesList),
                          Container(color: Colors.grey,height: 0.5,),
                        ],
                      );
                    },
                  ),
                ),
              ],
            )
          : Container(
              color: Colors.white,
              child: Column(
                children: [
                  Expanded(
                    flex: 1,
                    child: Align(
                      alignment: Alignment.bottomCenter,
                      child: InkWell(
                        onTap: () async {},
                        child: Image.asset(
                          StringUtils.IC_LOGO,
                          width: 50,
                          height: 50,
                        ),
                      ),
                    ),
                  ),
                  AddSizeBox(100, 0),
                  Expanded(
                    flex: 1,
                    child: Container(
                      child: Text(
                        StringUtils.CONTACT_SELECTION_WAIT_MESSAGE,
                        textAlign: TextAlign.center,
                        style: TextStyle(
                            decoration: TextDecoration.none,
                            fontSize: 15,
                            color: Colors.black),
                      ),
                    ),
                  ),
                ],
              ),
            ),
    );
  }

  void selectAll(bool isSelectAll) {
    for (int i = 0; i < _filterdContacts.length; i++) {
      _filterdContacts[i].isChecked = isSelectAll;
    }
  }

  onSearchTextChanged(String text) async {
    setState(() {
      _filterdContacts.clear();
      if (text.isEmpty) {
        _filterdContacts.addAll(_allContacts);
      } else
        _allContacts.forEach((userDetail) {
          var _phonesList = userDetail.contact.phones.toList();
          if (userDetail.contact.displayName
                  .toLowerCase()
                  .contains(text.toLowerCase()) ||
              (_phonesList.length > 0
                  ? _phonesList[0]
                      .value
                      .toLowerCase()
                      .contains(text.toLowerCase())
                  : false)) _filterdContacts.add(userDetail);
        });
    });
  }

  ListTile _buildListTile(CustomContact c, List<Item> list) {
    return ListTile(
      title: Text(c.contact.displayName ?? ""),
      subtitle: list.length >= 1 && list[0]?.value != null
          ? Text(list[0].value)
          : Text(''),
      trailing: Icon(
        Icons.check,
        color: c.isChecked ? Colors.green : Colors.transparent,
      ),
      onTap: () {
        setState(() {
          c.isChecked = !c.isChecked;
        });
      },
    );
  }

  refreshContacts() async {
    setState(() {
      _isLoading = true;
    });
    var contacts = await ContactsService.getContacts();
    _populateContacts(contacts);
  }

  void _populateContacts(Iterable<Contact> contacts) {
    _contacts = contacts.where((item) => item.displayName != null).toList();
    _contacts.sort((a, b) => a.displayName.compareTo(b.displayName));
    _allContacts =
        _contacts.map((contact) => CustomContact(contact: contact)).toList();
    setState(() {
      _uiCustomContacts.addAll(_allContacts);
      _filterdContacts.addAll(_allContacts);
      _isLoading = false;
    });
  }

  Widget AddTopButton(bool isSelectAll) {
    return Expanded(
      child: InkWell(
        onTap: () {
          setState(() {
            selectAll(isSelectAll);
          });
        },
        child: Container(
          height: 50,
          color: new ColorUtils().colorLightGray,
          child: Center(
            child: AddText(
                isSelectAll
                    ? StringUtils.SELECT_ALL
                    : StringUtils.UN_SELECT_ALL,
                Colors.blue,
                18,
                FontWeight.normal,
                false),
          ),
        ),
      ),
    );
  }

  Future<void> proceed() async {
    try {
      final result = await InternetAddress.lookup('google.com');
      if (result.isNotEmpty && result[0].rawAddress.isNotEmpty) {
        _uiCustomContacts = _filterdContacts
            .where((contact) => contact.isChecked == true)
            .toList();
        if (_uiCustomContacts != null && _uiCustomContacts.length > 0)
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                  builder: (context) =>
                      InviteContact(_uiCustomContacts, widget.bean)));
        else
          CommonUtils.showToast("Please Select Contact", Colors.black);
      }
    } on SocketException catch (_) {
      CommonUtils.showToast(StringUtils.CHECK_NETWORK, Colors.red);
    }
  }
}

class CustomContact {
  final Contact contact;
  bool isChecked;

  CustomContact({
    this.contact,
    this.isChecked = false,
  });
}
